import pygame
from   pygame.locals import *


from factories import Factory

from display import *


pygame.mixer.pre_init(22050,-16,2,2048)
pygame.init()


class MainLoop:
  def __init__(self):
    pygame.display.set_mode((500,550),0,32)
    pygame.display.set_caption('Hazard - PyWeek #11 "Caught"')
    
    self.screen = pygame.display.get_surface()
    
    self.game_screen = self.screen.subsurface([0,0,500,500])
    self.hud_screen  = self.screen.subsurface([0,500,500,50])
    
    
    self.factory = Factory()
    
    self.level = self.factory.generateWorld()
    
    
    self.controls = {'Up':False,'Down':False,'Left':False,
                     'Right':False,'Fire':False}
    
    self.paused = False
    
    self.frame_timer = pygame.time.Clock()
    
  
  def go(self):
    while True:
      self.frame_timer.tick(30)
      
      self.step()
      
    
  
  def step(self):
    for event in pygame.event.get():
      if event.type == QUIT:
        pygame.quit()
        exit()
      elif event.type == KEYDOWN:
        if event.key == K_ESCAPE:
          pygame.quit()
          exit()
        
        if event.key in (K_TAB,K_p):
          self.paused = not self.paused
        
        if event.key in (K_DOWN,K_s):
          self.controls['Down'] = True
        if event.key in (K_UP,K_w):
          self.controls['Up'] = True
        if event.key in (K_LEFT,K_a):
          self.controls['Left'] = True
        if event.key in (K_RIGHT,K_d):
          self.controls['Right'] = True
      elif event.type == KEYUP:
        if event.key in (K_DOWN,K_s):
          self.controls['Down'] = False
        if event.key in (K_UP,K_w):
          self.controls['Up'] = False
        if event.key in (K_LEFT,K_a):
          self.controls['Left'] = False
        if event.key in (K_RIGHT,K_d):
          self.controls['Right'] = False
        
    
    self.controls['Fire'] = pygame.mouse.get_pressed()[0]
    
    
    if not self.paused:
      self.level.update(self.controls)
    
    if self.level.player.health <= 0 or len(self.level.baddies) == 0:
      self.level = self.factory.generateWorld()
      
    
    drawGameScreen(self.level,self.game_screen,self.hud_screen)
    
    
  


if __name__ == '__main__':
  MainLoop().go()
  
