import pygame


import os


class DataManager:
  def __init__(self):
    self.data_dir = os.path.dirname(__file__)
    self.data_dir = os.path.join(self.data_dir,'..','data')
    self.data_dir = os.path.relpath(self.data_dir)
    
    self.images_loaded = {}
    self.sounds_loaded = {}
    
  
  def loadImage(self,img_path):
    img_path = os.path.join(self.data_dir,'image',img_path)
    
    img_path,_ = os.path.splitext(img_path)
    
    img_path += '.png'
    
    return pygame.image.load(img_path)
    
  
  def loadSound(self,snd_path):
    snd_path = os.path.join(self.data_dir,'sound',snd_path)
    
    snd_path,_ = os.path.splitext(img_path)
    
    snd_path += '.wav'
    
    return pygame.mixer.Sound(snd_path)
    
  
  def loadPlayerImage(self):
    if 'Player' in self.images_loaded:
      return self.images_loaded['Player']
      
    
    image = self.loadImage(os.path.join('agent','Player'))
    
    self.images_loaded['Player'] = image
    
    return image
    
  
  def loadBaddieImage(self,level):
    if 'Baddie' in self.images_loaded:
      if self.images_loaded['Baddie'][level] != None:
        return self.images_loaded['Baddie'][level]
        
    else:
      self.images_loaded['Baddie'] = [None,None,None]
    
    image = self.loadImage(os.path.join('agent',('Baddie-'+str(level))))
    
    self.images_loaded['Baddie'][level] = image
    
    return image
    
  
  def loadProjectileImage(self,name,level=0):
    if 'Projectile' in self.images_loaded:
      proj_imgs = self.images_loaded['Projectile']
      
      if name == 'Player' and proj_imgs[name] != None:
        return proj_imgs['Player']
      elif name == 'Baddie' and proj_imgs[name][level] != None:
        return proj_imgs['Baddie'][level]
      
    else:
      self.images_loaded['Projectile'] = {'Player':None,'Baddie':[None,None,None]}
      
    
    image = None
    
    if name == 'Player':
      image = self.loadImage(os.path.join('projectile','Player'))
      self.images_loaded['Projectile']['Player'] = image
      
    else:
      image = self.loadImage(os.path.join('projectile',('Baddie-'+str(level))))
      self.images_loaded['Projectile']['Baddie'][level] = image
    
    return image
    
  
  def loadTerrainImage(self,name):
    if 'Terrain' in self.images_loaded:
      if self.images_loaded['Terrain'][name] != None:
        return self.images_loaded['Terrain'][name]
      
    else:
      self.images_loaded['Terrain'] = {'Wall':None,'Floor':None}
    
    image = self.loadImage(os.path.join('terrain',name))
    
    self.images_loaded['Terrain'][name] = image
    
    return image
    
  
