import pygame

import math

from util import clamp_off


class Entity(object):
  def __init__(self,pos,size,angle,image):
    self.rect = pygame.Rect(0,0,size,size)
    
    self.rect.center = pos
    
    self.heading = angle
    
    self.image = image
    
    self.world = None
    
  
  
  def getImage(self):
    image = self.image
    image = pygame.transform.rotozoom(image,self.heading,1.0)
    
    return image
    
  
  def getBlitInfo(self):
    image     = self.getImage()
    blit_rect = image.get_rect()
    
    blit_rect.center = self.rect.center
    
    return image,blit_rect
    
  


class Projectile(Entity):
  def __init__(self,origin,size,angle,speed,damage,image):
    super(Projectile,self).__init__(origin,size,angle,image)
    
    self.damage = damage;
    
    self.velocity = [0,0]
    
    self.velocity[0] = math.cos(math.radians(angle)) * speed
    self.velocity[1] = math.sin(math.radians(angle)) * speed
    
    self.velocity[1] *= -1
    
    self.alive     = True
    self.exploding = False
    
  
  def update(self):
    if self.alive:
      self.rect.left += self.velocity[0]
      self.rect.top  += self.velocity[1]
      
      if not self.rect.colliderect(self.world.area):
        self.alive = False
        return None
        
      
      for block in self.world.massive_terrain:
        if self.rect.colliderect(block.rect):
          self.alive = False
          #self.exploding = True
          
          return None
        
      
      targets = []
      
      if self in self.world.projectiles['Player']:
        targets = self.world.baddies
      else:
        targets = [self.world.player]
      
      for dude in targets:
        if self.rect.colliderect(dude.rect):
          dude.health -= self.damage
          
          if dude.health >= 0:
            self.alive = False
            
            return None
          
        
      
    
  


class Agent(Entity):
  def __init__(self,pos,size,angle,health,speed,image,shot_image):
    super(Agent,self).__init__(pos,size,angle,image)
    
    self.shot_image = shot_image
    
    self.health = health
    
    self.speed  = speed
    
    self.velocity = [0,0]
    
    self.collisions = {'Up':False,'Down':False,'Left':False,'Right':False}
    
  
  def attack(self):
    pass
  
  def move(self,dx,dy):
    self.collisions[ 'Left'] = False
    self.collisions['Right'] = False
    self.collisions[   'Up'] = False
    self.collisions[ 'Down'] = False
      
    
    dx = int(dx)
    dy = int(dy)
    
    dudes = self.world.baddies + [self.world.player]
    
    dudes.remove(self)
    
    self.rect.left += dx
    
    for block in self.world.massive_terrain + dudes:
      if self.rect.colliderect(block.rect):
        coll_direction = clamp_off(self.rect,block.rect,dx,0)
        
        if coll_direction != 'None':
          self.collisions[coll_direction] = True
        
      
    
    self.rect.top += dy
    
    for block in self.world.massive_terrain + dudes:
      if self.rect.colliderect(block.rect):
        coll_direction = clamp_off(self.rect,block.rect,0,dy)
        
        if coll_direction != 'None':
          self.collisions[coll_direction] = True
        
      
    
    
    
    if self.rect.left < self.world.area.left:
      self.collisions['Left'] = True
    if self.rect.right > self.world.area.right:
      self.collisions['Right'] = True
    
    if self.rect.top < self.world.area.top:
      self.collisions['Up'] = True
    if self.rect.bottom > self.world.area.bottom:
      self.collisions['Down'] = True
    
    self.rect.clamp_ip(self.world.area)
    
  
