import pygame

from entities import *

from player   import *
from baddie   import *

from terrain  import *
from world    import *

from data     import *


import random


class Factory:
  def __init__(self):
    self.data_manager = DataManager()
    
  
  def makePlayer(self,pos):
    return Player(pos,0,self.data_manager.loadPlayerImage(),
                        self.data_manager.loadProjectileImage('Player'))
    
  
  def makeBaddie(self,pos,level):
    return Baddie(pos,0,level,self.data_manager.loadBaddieImage(level),
                    self.data_manager.loadProjectileImage('Baddie',level))
    
  
  def makeTerrainTile(self,pos,name):
    if name == 'Wall':
      return TerrainTile(pos,True,self.data_manager.loadTerrainImage('Wall'))
    else:
      return TerrainTile(pos,False,self.data_manager.loadTerrainImage('Floor'))
      
  
  def makeTestWorld(self):
    player = self.makePlayer([25,25])
    
    terrain = []
    baddies = []
    
    for row in range(0,2):
      for column in range(0,10):
        terrain.append(self.makeTerrainTile([column,row],'Floor'))
    
    for row in range(3,6):
      for column in range(0,10):
        terrain.append(self.makeTerrainTile([column,row],'Floor'))
    
    for row in range(3,6):
      for column in range(0,10):
        terrain.append(self.makeTerrainTile([column,row],'Floor'))
    
    for row in range(8,10):
      for column in range(0,10):
        terrain.append(self.makeTerrainTile([column,row],'Floor'))
    
    for row in range(2,3):
      for column in range(3,8):
        terrain.append(self.makeTerrainTile([column,row],'Wall'))
      
      for column in range(0,3):
        terrain.append(self.makeTerrainTile([column,row],'Floor'))
      
      for column in range(8,10):
        terrain.append(self.makeTerrainTile([column,row],'Floor'))
    
    for row in range(6,8):
      for column in range(2,6):
        terrain.append(self.makeTerrainTile([column,row],'Wall'))
      
      for column in range(0,2):
        terrain.append(self.makeTerrainTile([column,row],'Floor'))
      
      for column in range(6,10):
        terrain.append(self.makeTerrainTile([column,row],'Floor'))
    
    baddies.append(self.makeBaddie([325,375],0))
    baddies.append(self.makeBaddie([425,225],0))
    
    return GameWorld(player,terrain,baddies)
    
  
  def generateWorld(self):
    terrain = []
    baddies = []
    
    walls = []
    
    for column in range(10):
      walls.append([])
      for row in range(10):
        walls[-1].append(False)
      
    
    player = self.makePlayer([25,475])
    
    for wall_number in range(random.randint(4,6)):
      wall_pos = [random.randint(1,4),random.randint(1,4)]
      
      wall_pos[0] *= 2
      wall_pos[1] *= 2
      
      wall_direction = random.choice(['Left','Right','Up','Down'])
      
      if wall_pos[0] <= 1 and wall_direction == 'Left':
        wall_direction = 'Right'
      
      if wall_pos[0] >= 8 and wall_direction == 'Right':
        wall_direction = 'Left'
      
      if wall_pos[1] <= 1 and wall_direction == 'Up':
        wall_direction = 'Down'
      
      if wall_pos[1] >= 8 and wall_direction == 'Down':
        wall_direction = 'Up'
      
      wall_length = random.randint(3,4)
      
      for index in range(wall_length):
        walls[wall_pos[0]][wall_pos[1]] = True
        
        if wall_direction == 'Left':
          wall_pos[0] -= 1
        if wall_direction == 'Right':
          wall_pos[0] += 1
        if wall_direction == 'Up':
          wall_pos[1] -= 1
        if wall_direction == 'Down':
          wall_pos[1] += 1
        
        if (wall_pos[0] <= 0 or wall_pos[1] <= 0 or
            wall_pos[0] >= 9 or wall_pos[1] >= 9):
          break
        
    
    for baddie_number in range(random.randint(4,8)):
      baddie_start = [random.randint(1,9),random.randint(1,9)]
      
      while walls[baddie_start[0]][baddie_start[1]]:
        baddie_start = [random.randint(1,9),random.randint(1,9)]
      
      baddie_start[0] *= 50
      baddie_start[1] *= 50
      
      baddie_start[0] += 25
      baddie_start[1] += 25
      
      baddie_level = 0
      
      if random.random() < 0.3:
        baddie_level = 1
      elif random.random() < 0.05:
        baddie_level = 2
      
      baddies.append(self.makeBaddie(baddie_start,baddie_level))
      
    
    #print walls
    
    for column in range(10):
      for row in range(10):
        #print walls[row]
        
        if walls[column][row]:
          terrain.append(self.makeTerrainTile([column,row],'Wall'))
          
          #print [column,row]
        else:
          terrain.append(self.makeTerrainTile([column,row],'Floor'))
        
    
    return GameWorld(player,terrain,baddies)
    
  
